@@ -107,6 +107,69 @@ function jsGetDataTableHeightPx() {
     return retHeightPx;
 }
 
+/**
+ * refresh table data without full reload of page
+ *
+ * Note: this is only for client based table data (serverside = False)
+ *
+ * @param table - table to redraw
+ * @param resturl - url to use to retrieve updated data for table
+ * @param paging - see https://datatables.net/reference/api/draw() (default 'full-reset')
+ */
+function refresh_table_data(table, resturl, paging) {
+    // (dataTables default) ordering and search will be recalculated and the rows redrawn in their new positions.
+    // The paging will be reset back to the first page.
+    if (paging === undefined) {
+        paging = 'full-reset';
+    }
+
+    var rowId = table.settings()[0].rowId;
+
+    // save {rowid: row, } for all current data
+    var currrows = {};
+    var rowndxs = table.rows()[0];
+    for (var i=0; i<rowndxs.length; i++) {
+        var rowndx = rowndxs[i];
+        currrows[table.row(rowndx).id()] = table.row(rowndx);
+    }
+
+    var jqxhr = $.get(resturl, function( respdata ) {
+        for (var i=0; i<respdata.length; i++) {
+            var resprow = respdata[i];
+            // replace row if it exists
+            if (currrows[resprow[rowId]] !== undefined) {
+                table.row('#' + resprow[rowId]).data(resprow);
+                delete currrows[resprow[rowId]];
+            // add row if it is new
+            } else {
+                table.row.add(resprow);
+            }
+        }
+
+        // any remaining rows in currrows need to be deleted
+        $.each(currrows, function(k, v){
+            table.row('#' + k).remove();
+        });
+
+        // redraw
+        table.draw(paging);
+    });
+}
+
+/**
+ * return a grip for rowReorder function
+ *
+ * @returns {string}
+ */
+function render_grip(data, type, row) {
+    // see https://datatables.net/forums/discussion/comment/187037/#Comment_187037
+    if ( type === 'display' ) {
+        return '<i class="fa fa-grip-horizontal" aria-hidden="true"></i>';
+    }
+    return data;
+}
+
+
 /**
  * get the button options, correctly annotated with indicated editor
  *
@@ -172,6 +235,36 @@ function render_icon(iconclass) {
     return fn;
 }
 
+/**
+ * use in yadcf options as custom_func parameter, along with filter_type = 'date_custom_func'
+ * @param fromdateattr
+ * @param todateattr
+ * @returns {function(filterVal, columnVal, rowValues, stateVal): boolean}
+ */
+function yadcf_between_dates(fromdateattr, todateattr) {
+    function yadcf_between_dates_fn(filterVal, columnVal, rowValues, stateVal) {
+        console.log('filterVal='+filterVal);
+        console.log('columnVal='+columnVal);
+        console.log('rowValues='+rowValues);
+        console.log('stateVal='+stateVal);
+
+        // no filter means return all records
+        if (filterVal === '') {
+          return true;
+        }
+
+        // blank fromdate is from the beginning of time, blank todate is to the end of time
+        if (    (stateVal[fromdateattr] === '' || filterVal >= stateVal[fromdateattr])
+             && (stateVal[todateattr] === ''   || filterVal <= stateVal[todateattr])) {
+            return true;
+
+        } else {
+          return false;
+        }
+    }
+    return yadcf_between_dates_fn;
+}
+
 /**
  * create click event handler for indicated datatable, cell
  * when cell clicked, row is selected and button_name'd button is triggered (pressed)
